@@ -6,7 +6,7 @@
 
    libquantum is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
-   by the Free Software Foundation; either version 2 of the License,
+   by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    libquantum is distributed in the hope that it will be useful, but
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with libquantum; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301, USA
 
 */
 
@@ -37,7 +37,7 @@ struct quantum_matrix_struct {
 
 typedef struct quantum_matrix_struct quantum_matrix;
 
-#define M(m,x,y) m.t[x+y*m.cols]
+#define M(m,x,y) m.t[(x)+(y)*m.cols]
 
 extern unsigned long quantum_memman(long change);
 
@@ -45,4 +45,6 @@ extern quantum_matrix quantum_new_matrix(int cols, int rows);
 extern void quantum_delete_matrix(quantum_matrix *m);
 extern void quantum_print_matrix(quantum_matrix m);
 
+extern quantum_matrix quantum_mmult(quantum_matrix A, quantum_matrix B);
+
 #endif
